# You need to create the external data group before you run the script:
#
# Create dummy file:
# echo "network 169.254.1.1/32 := \"\"," > /var/tmp/cs-ip.txt
# Create external data group with dummy file as source:
# tmsh create ltm data-group external ext-crowdsec-blaclist description "Crowdsec blacklist" type ip source-path file:/var/tmp/cs-ip.txt
#
# Here is an example of an iRule using the external data group:
# when HTTP_REQUEST {
#    if { [class match [IP::client_addr] equals "ext-crowdsec-blaclist" ] } {
#        HTTP::respond 200 content "Blacklisted......"
#    }
#}

# Assigning variables for the different files being used
ALL_IPS="/var/tmp/all.txt"
IPV4="/var/tmp/ipv4.txt"
IPV6="/var/tmp/ipv6.txt"
AFM="/var/tmp/afm.txt"
AFM_SPLIT="/var/tmp/afm_split.txt"
AFM_ACCESS_LIST="crowdsec-blacklist"
EXT_DG_NAME="ext-crowdsec-blaclist"
EXT_DG="/var/tmp/extdg.txt"
LTMLOG="/var/log/ltm"

# URL for the Crowdsec Blacklist Mirror endpoint
URL="http://10.10.10.10:41412/security/blocklist"

# To reduce load on the mcpd daemon during import, this split number is the amount of IP's imported into the address list at a time
SPLIT="25"

# Set where you want the CS Blacklist to be populated - Data Group (USE_DG), AFM address List (USE_AFM)
USE_DG="yes"
USE_AFM="yes"

DATE_START=$(date)
echo "$DATE_START Crowdsec Blacklist Update started..." >> $LTMLOG

# Download the complete list
statuscode=$(curl --silent --write-out "%{http_code}" -o $ALL_IPS $URL)
if test $statuscode -ne 200; then
    echo "Error downloading Crowdsec Blacklist, statuscode: $statuscode" >>/var/log/ltm 2>&1
    return 1
fi

# Split the IPv4 and IPv6 addresses into different files. We need to manipulate the IPv6 and apply a prefix CIDR files before we can import them
grep -E -o "([0-9]{1,3}[\.]){3}[0-9]{1,3}" $ALL_IPS > $IPV4
grep -v -E  "([0-9]{1,3}[\.]){3}[0-9]{1,3}" $ALL_IPS > $IPV6

# Import the list into an external data group
if [[ $USE_DG == "yes" ]]
then
       # Change the format to the format external data groups uses
       for i in $(cat $IPV4)
       do
              echo "network $i/32 := \"\", " >> $EXT_DG
       done
       
       # For IPv6 addresses having a digit in the end we apply a host prefix (/128)
       for i in $(cat $IPV6|grep '[0-9]$')
       do
              echo "network $i/128 := \"\", " >> $EXT_DG
       done
       
       # For IPv6 addresses that are condensed we apply network prefix (/64)
       for i in $(cat $IPV6|grep -v '[0-9]$')
       do
              echo "network $i/64 := \"\", " >> $EXT_DG
       done
       
       # Do the actual import
       tmsh modify ltm data-group external $EXT_DG_NAME source-path file:$EXT_DG
fi

# Import the list into an address list
if [[ $USE_AFM == "yes" ]]
then
       # Address list require the IPs to be on the same line, so we remove all newline characters
       tr '\n' ' ' < $ALL_IPS > $AFM
       
       # To reduce load on mcpd during import, we split the list into smaller chunks of IPs defined by the SPLIT variable
       cat $AFM |xargs -n$SPLIT> $AFM_SPLIT
       
       # Prep percentage values for the log output 
       s=$(cat $AFM_SPLIT|wc -l)
       P25=$(( s*25/100 ))
       P50=$(( s*50/100 ))
       P75=$(( s*75/100 ))
       P99=$(( s*99/100 ))
       
       # Empty the list by inserting a dummy IP
       tmsh modify security firewall address-list $AFM_ACCESS_LIST { addresses replace-all-with { 169.254.1.1 }}
       
       for i in $(seq $s)
       do 
           # Import the list line by line from the splitted file   
           tmsh modify security firewall address-list $AFM_ACCESS_LIST { addresses add { $(awk -v a=$i '{if(NR==a) print $0}' $AFM_SPLIT) }}
           DATE=$(date)
           # Output indication of progress into the ltm log, as the import can take several minutes
           if ((i == P25)); then echo "$DATE Crowdsec Address List import 25% done" >> $LTMLOG; fi
           if ((i == P50)); then echo "$DATE Crowdsec Address List import 50% done" >> $LTMLOG; fi
           if ((i == P75)); then echo "$DATE Crowdsec Address List import 75% done" >> $LTMLOG; fi
           if ((i == P99)); then echo "$DATE Crowdsec Address List import 99% done" >> $LTMLOG; fi
       done
       
       # Remove dummy IP from the address list
       tmsh modify security firewall address-list $AFM_ACCESS_LIST { addresses delete { 169.254.1.1 }}
fi

# Do a count of IPs and conclude the import with a total figure in the ltm log
CS_IPV4=$(wc -l < $IPV4)
CS_IPV6=$(wc -l < $IPV6)
DATE_END=$(date)
echo "$DATE_END Crowdsec Blocklist Updated with $CS_IPV4 IPv4 addresses and $CS_IPV6 IPv6 addresses" >> $LTMLOG

# Clean up files
rm -f $ALL_IPS $IPV4 $IPV6 $EXT_DG $AFM $AFM_SPLIT
